@@ -25,23 +25,26 @@ from typing import Any, Dict
 from sphinx.application import Sphinx
 from sphinx.ext.autodoc import AttributeDocumenter, MethodDocumenter
 
-from .dbus_proxy_async_interfaces import DbusInterfaceCommonAsync
-from .dbus_proxy_async_method import DbusMethodAsync
-from .dbus_proxy_async_property import DbusPropertyAsync
-from .dbus_proxy_async_signal import DbusSignalAsync
+from .dbus_proxy_async_method import DbusMethodAsyncBinded
+from .dbus_proxy_async_property import (
+    DbusPropertyAsync,
+    DbusPropertyAsyncBinded,
+)
+from .dbus_proxy_async_signal import DbusSignalAsync, DbusSignalBinded
 
 
 class DbusMethodDocumenter(MethodDocumenter):
 
-    objtype = 'DbusMethodAsync'
+    objtype = 'DbusMethodAsyncBinded'
     directivetype = 'method'
     priority = 100 + MethodDocumenter.priority
 
     @classmethod
     def can_document_member(cls, member: Any, *args: Any) -> bool:
-        return isinstance(member, DbusMethodAsync)
+        return isinstance(member, DbusMethodAsyncBinded)
 
     def import_object(self, raiseerror: bool = False) -> bool:
+        self.objpath.append('dbus_method')
         self.objpath.append('original_method')
         ret = super().import_object(raiseerror)
         self.objpath.pop()
@@ -61,62 +64,82 @@ class DbusMethodDocumenter(MethodDocumenter):
 
 class DbusPropertyDocumenter(AttributeDocumenter):
 
-    objtype = 'DbusPropertyAsync'
+    objtype = 'DbusPropertyAsyncBinded'
     directivetype = 'attribute'
     priority = 100 + AttributeDocumenter.priority
 
     @classmethod
     def can_document_member(cls, member: Any, *args: Any) -> bool:
-        return isinstance(member, DbusPropertyAsync)
+        return isinstance(member, DbusPropertyAsyncBinded)
 
-    def update_annotations(self,
-                           parent: DbusInterfaceCommonAsync) -> None:
+    def import_object(self, raiseerror: bool = False) -> bool:
 
-        assert isinstance(self.object, DbusPropertyAsync)
-        property_annotation = \
-            self.object.property_getter.__annotations__['return']
-
-        parent.__annotations__[self.object_name] = property_annotation
+        self.objpath.append('dbus_property')
+        ret = super().import_object(raiseerror)
+        self.objpath.pop()
+        return ret
 
     def add_content(self,
                     *args: Any, **kwargs: Any,
                     ) -> None:
 
+        assert isinstance(self.object, DbusPropertyAsync)
+        python_type = self.object.property_getter.__annotations__['return']
+        dbus_type = self.object.property_signature
+
         source_name = self.get_sourcename()
         self.add_line('', source_name)
         self.add_line('**D-Bus property**', source_name)
         self.add_line('', source_name)
+        self.add_line(
+            f"**Python type**: *{python_type}*", source_name
+        )
+        self.add_line('', source_name)
+        self.add_line(
+            f"**D-Bus type**: {dbus_type}", source_name
+        )
+        self.add_line('', source_name)
 
         super().add_content(*args, **kwargs)
 
 
 class DbusSignalDocumenter(AttributeDocumenter):
 
-    objtype = 'DbusSignal'
+    objtype = 'DbusSignalBinded'
     directivetype = 'attribute'
     priority = 100 + AttributeDocumenter.priority
 
     @classmethod
     def can_document_member(cls, member: Any, *args: Any) -> bool:
-        return isinstance(member, DbusSignalAsync)
+        return isinstance(member, DbusSignalBinded)
 
-    def update_annotations(self,
-                           parent: DbusInterfaceCommonAsync) -> None:
+    def import_object(self, raiseerror: bool = False) -> bool:
 
-        assert isinstance(self.object, DbusSignalAsync)
-        signal_annotation = \
-            self.object.__annotations__['return']
-
-        parent.__annotations__[self.object_name] = signal_annotation
+        self.objpath.append('dbus_signal')
+        ret = super().import_object(raiseerror)
+        self.objpath.pop()
+        return ret
 
     def add_content(self,
                     *args: Any, **kwargs: Any,
                     ) -> None:
 
+        assert isinstance(self.object, DbusSignalAsync)
+        python_type = self.object.__annotations__['return']
+        dbus_type = self.object.signal_signature
+
         source_name = self.get_sourcename()
         self.add_line('', source_name)
         self.add_line('**D-Bus signal**', source_name)
         self.add_line('', source_name)
+        self.add_line(
+            f"**Python type**: *{python_type}*", source_name
+        )
+        self.add_line('', source_name)
+        self.add_line(
+            f"**D-Bus type**: {dbus_type}", source_name
+        )
+        self.add_line('', source_name)
 
         super().add_content(*args, **kwargs)
 
