@@ -18,6 +18,7 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */
+#include <errno.h>
 #include "sd_bus_internals.h"
 
 static void SdBus_dealloc(SdBusObject* self) {
@@ -28,18 +29,17 @@ static void SdBus_dealloc(SdBusObject* self) {
 }
 
 static int SdBus_init(SdBusObject* self, PyObject* Py_UNUSED(args), PyObject* Py_UNUSED(kwds)) {
-        self->sd_bus_ref = NULL;
-        self->reader_fd = NULL;
+        CALL_SD_BUS_CHECK_RETURN_NEG1(sd_bus_new(&(self->sd_bus_ref)));
         return 0;
 }
 
 #ifndef Py_LIMITED_API
 static SdBusMessageObject* SdBus_new_method_call_message(SdBusObject* self, PyObject* const* args, Py_ssize_t nargs) {
         SD_BUS_PY_CHECK_ARGS_NUMBER(4);
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(1, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(2, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(3, PyUnicode_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(1, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(2, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(3, PyUnicode_Check);
 
         const char* destination_bus_name = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[0]);
         const char* object_path = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[1]);
@@ -54,7 +54,7 @@ static SdBusMessageObject* SdBus_new_method_call_message(SdBusObject* self, PyOb
         CALL_PYTHON_BOOL_CHECK(PyArg_ParseTuple(args, "ssss", &destination_bus_name, &object_path, &interface_name, &member_name, NULL));
 #endif
         SdBusMessageObject* new_message_object CLEANUP_SD_BUS_MESSAGE =
-            (SdBusMessageObject*)CALL_PYTHON_AND_CHECK(PyObject_CallFunctionObjArgs(SdBusMessage_class, NULL));
+            (SdBusMessageObject*)CALL_PYTHON_AND_CHECK(SD_BUS_PY_CLASS_DUNDER_NEW(SdBusMessage_class));
 
         CALL_SD_BUS_AND_CHECK(
             sd_bus_message_new_method_call(self->sd_bus_ref, &new_message_object->message_ref, destination_bus_name, object_path, interface_name, member_name));
@@ -66,10 +66,10 @@ static SdBusMessageObject* SdBus_new_method_call_message(SdBusObject* self, PyOb
 #ifndef Py_LIMITED_API
 static SdBusMessageObject* SdBus_new_property_get_message(SdBusObject* self, PyObject* const* args, Py_ssize_t nargs) {
         SD_BUS_PY_CHECK_ARGS_NUMBER(4);
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(1, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(2, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(3, PyUnicode_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(1, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(2, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(3, PyUnicode_Check);
 
         const char* destination_service_name = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[0]);
         const char* object_path = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[1]);
@@ -84,7 +84,7 @@ static SdBusMessageObject* SdBus_new_property_get_message(SdBusObject* self, PyO
         CALL_PYTHON_BOOL_CHECK(PyArg_ParseTuple(args, "ssss", &destination_service_name, &object_path, &interface_name, &property_name, NULL));
 #endif
         SdBusMessageObject* new_message_object CLEANUP_SD_BUS_MESSAGE =
-            (SdBusMessageObject*)CALL_PYTHON_AND_CHECK(PyObject_CallFunctionObjArgs(SdBusMessage_class, NULL));
+            (SdBusMessageObject*)CALL_PYTHON_AND_CHECK(SD_BUS_PY_CLASS_DUNDER_NEW(SdBusMessage_class));
         CALL_SD_BUS_AND_CHECK(sd_bus_message_new_method_call(self->sd_bus_ref, &new_message_object->message_ref, destination_service_name, object_path,
                                                              "org.freedesktop.DBus.Properties", "Get"));
 
@@ -99,10 +99,10 @@ static SdBusMessageObject* SdBus_new_property_get_message(SdBusObject* self, PyO
 #ifndef Py_LIMITED_API
 static SdBusMessageObject* SdBus_new_property_set_message(SdBusObject* self, PyObject* const* args, Py_ssize_t nargs) {
         SD_BUS_PY_CHECK_ARGS_NUMBER(4);
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(1, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(2, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(3, PyUnicode_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(1, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(2, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(3, PyUnicode_Check);
 
         const char* destination_service_name = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[0]);
         const char* object_path = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[1]);
@@ -117,7 +117,7 @@ static SdBusMessageObject* SdBus_new_property_set_message(SdBusObject* self, PyO
         CALL_PYTHON_BOOL_CHECK(PyArg_ParseTuple(args, "ssss", &destination_service_name, &object_path, &interface_name, &property_name, NULL));
 #endif
         SdBusMessageObject* new_message_object CLEANUP_SD_BUS_MESSAGE =
-            (SdBusMessageObject*)CALL_PYTHON_AND_CHECK(PyObject_CallFunctionObjArgs(SdBusMessage_class, NULL));
+            (SdBusMessageObject*)CALL_PYTHON_AND_CHECK(SD_BUS_PY_CLASS_DUNDER_NEW(SdBusMessage_class));
         CALL_SD_BUS_AND_CHECK(sd_bus_message_new_method_call(self->sd_bus_ref, &new_message_object->message_ref, destination_service_name, object_path,
                                                              "org.freedesktop.DBus.Properties", "Set"));
 
@@ -132,9 +132,9 @@ static SdBusMessageObject* SdBus_new_property_set_message(SdBusObject* self, PyO
 #ifndef Py_LIMITED_API
 static SdBusMessageObject* SdBus_new_signal_message(SdBusObject* self, PyObject* const* args, Py_ssize_t nargs) {
         SD_BUS_PY_CHECK_ARGS_NUMBER(3);
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);  // Path
-        SD_BUS_PY_CHECK_ARG_TYPE(1, PyUnicode_Type);  // Interface
-        SD_BUS_PY_CHECK_ARG_TYPE(2, PyUnicode_Type);  // Member
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);  // Path
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(1, PyUnicode_Check);  // Interface
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(2, PyUnicode_Check);  // Member
 
         const char* object_path = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[0]);
         const char* interface_name = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[1]);
@@ -147,7 +147,7 @@ static SdBusMessageObject* SdBus_new_signal_message(SdBusObject* self, PyObject*
         CALL_PYTHON_BOOL_CHECK(PyArg_ParseTuple(args, "sss", &object_path, &interface_name, &member_name, NULL));
 #endif
         SdBusMessageObject* new_message_object CLEANUP_SD_BUS_MESSAGE =
-            (SdBusMessageObject*)CALL_PYTHON_AND_CHECK(PyObject_CallFunctionObjArgs(SdBusMessage_class, NULL));
+            (SdBusMessageObject*)CALL_PYTHON_AND_CHECK(SD_BUS_PY_CLASS_DUNDER_NEW(SdBusMessage_class));
 
         CALL_SD_BUS_AND_CHECK(sd_bus_message_new_signal(self->sd_bus_ref, &new_message_object->message_ref, object_path, interface_name, member_name));
 
@@ -172,7 +172,7 @@ static SdBusMessageObject* SdBus_call(SdBusObject* self, PyObject* args) {
         CALL_PYTHON_BOOL_CHECK(PyArg_ParseTuple(args, "O", &call_message, NULL));
 #endif
         SdBusMessageObject* reply_message_object CLEANUP_SD_BUS_MESSAGE =
-            (SdBusMessageObject*)CALL_PYTHON_AND_CHECK(PyObject_CallFunctionObjArgs(SdBusMessage_class, NULL));
+            (SdBusMessageObject*)CALL_PYTHON_AND_CHECK(SD_BUS_PY_CLASS_DUNDER_NEW(SdBusMessage_class));
 
         sd_bus_error error __attribute__((cleanup(sd_bus_error_free))) = SD_BUS_ERROR_NULL;
 
@@ -210,9 +210,9 @@ int future_set_exception_from_message(PyObject* future, sd_bus_message* message)
 
         PyObject* exception_to_raise = PyDict_GetItemWithError(dbus_error_to_exception_dict, error_name_str);
 
-        PyObject* exception_occured = PyErr_Occurred();
-        if (exception_occured) {
-                Py_XDECREF(CALL_PYTHON_CHECK_RETURN_NEG1(PyObject_CallMethodObjArgs(future, set_exception_str, exception_occured, NULL)));
+        PyObject* exception_occurred = PyErr_Occurred();
+        if (exception_occurred) {
+                Py_XDECREF(CALL_PYTHON_CHECK_RETURN_NEG1(PyObject_CallMethodObjArgs(future, set_exception_str, exception_occurred, NULL)));
                 return 0;
         }
 
@@ -237,10 +237,12 @@ static PyObject* SdBus_get_fd(SdBusObject* self, PyObject* Py_UNUSED(args)) {
         return PyLong_FromLong((long)file_descriptor);
 }
 
-#define CHECK_SD_BUS_READER            \
-        if (self->reader_fd == NULL) { \
-                register_reader(self); \
-        }
+#define CHECK_SD_BUS_READER                                             \
+        ({                                                              \
+                if (self->reader_fd == NULL) {                          \
+                        CALL_PYTHON_EXPECT_NONE(register_reader(self)); \
+                }                                                       \
+        })
 
 PyObject* register_reader(SdBusObject* self) {
         PyObject* running_loop CLEANUP_PY_OBJECT = CALL_PYTHON_AND_CHECK(PyObject_CallFunctionObjArgs(asyncio_get_running_loop, NULL));
@@ -262,12 +264,17 @@ static PyObject* SdBus_drive(SdBusObject* self, PyObject* Py_UNUSED(args)) {
         int return_value = 1;
         while (return_value > 0) {
                 return_value = sd_bus_process(self->sd_bus_ref, NULL);
-                if (return_value == -104)  // -ECONNRESET
-                {
+                if (return_value < 0) {
                         CALL_PYTHON_AND_CHECK(unregister_reader(self));
-                        Py_RETURN_NONE;
+                        if (-ECONNRESET == return_value) {
+                                // Connection gracefully terminated
+                                Py_RETURN_NONE;
+                        } else {
+                                // Error occurred processing sdbus
+                                CALL_SD_BUS_AND_CHECK(return_value);
+                                return NULL;
+                        }
                 }
-                CALL_SD_BUS_AND_CHECK(return_value);
 
                 if (PyErr_Occurred()) {
                         return NULL;
@@ -291,7 +298,7 @@ int SdBus_async_callback(sd_bus_message* m,
         if (!sd_bus_message_is_method_error(m, NULL)) {
                 // Not Error, set Future result to new message object
 
-                SdBusMessageObject* reply_message_object CLEANUP_SD_BUS_MESSAGE = (SdBusMessageObject*)PyObject_CallFunctionObjArgs(SdBusMessage_class, NULL);
+                SdBusMessageObject* reply_message_object CLEANUP_SD_BUS_MESSAGE = (SdBusMessageObject*)SD_BUS_PY_CLASS_DUNDER_NEW(SdBusMessage_class);
                 if (reply_message_object == NULL) {
                         return -1;
                 }
@@ -325,7 +332,7 @@ static PyObject* SdBus_call_async(SdBusObject* self, PyObject* args) {
 
         PyObject* new_future = CALL_PYTHON_AND_CHECK(PyObject_CallMethod(running_loop, "create_future", ""));
 
-        SdBusSlotObject* new_slot_object CLEANUP_SD_BUS_SLOT = (SdBusSlotObject*)CALL_PYTHON_AND_CHECK(PyObject_CallFunctionObjArgs(SdBusSlot_class, NULL));
+        SdBusSlotObject* new_slot_object CLEANUP_SD_BUS_SLOT = (SdBusSlotObject*)CALL_PYTHON_AND_CHECK(SD_BUS_PY_CLASS_DUNDER_NEW(SdBusSlot_class));
 
         CALL_SD_BUS_AND_CHECK(
             sd_bus_call_async(self->sd_bus_ref, &new_slot_object->slot_ref, call_message->message_ref, SdBus_async_callback, new_future, (uint64_t)0));
@@ -345,8 +352,8 @@ static int _check_is_sdbus_interface(PyObject* type_to_check) {
 static PyObject* SdBus_add_interface(SdBusObject* self, PyObject* const* args, Py_ssize_t nargs) {
         SD_BUS_PY_CHECK_ARGS_NUMBER(3);
         SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, _check_is_sdbus_interface);
-        SD_BUS_PY_CHECK_ARG_TYPE(1, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(2, PyUnicode_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(1, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(2, PyUnicode_Check);
 
         SdBusInterfaceObject* interface_object = (SdBusInterfaceObject*)args[0];
         const char* path_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[1]);
@@ -371,7 +378,7 @@ static PyObject* SdBus_add_interface(SdBusObject* self, PyObject* args) {
 int _SdBus_signal_callback(sd_bus_message* m, void* userdata, sd_bus_error* Py_UNUSED(ret_error)) {
         PyObject* async_queue = userdata;
 
-        SdBusMessageObject* new_message_object CLEANUP_SD_BUS_MESSAGE = (SdBusMessageObject*)PyObject_CallFunctionObjArgs(SdBusMessage_class, NULL);
+        SdBusMessageObject* new_message_object CLEANUP_SD_BUS_MESSAGE = (SdBusMessageObject*)SD_BUS_PY_CLASS_DUNDER_NEW(SdBusMessage_class);
         if (new_message_object == NULL) {
                 return -1;
         }
@@ -412,17 +419,23 @@ int _SdBus_match_signal_instant_callback(sd_bus_message* m, void* userdata, sd_b
 }
 
 #ifndef Py_LIMITED_API
+
+static int _unicode_or_none(PyObject* some_object) {
+        return (PyUnicode_Check(some_object) || (Py_None == some_object));
+}
+
 static PyObject* SdBus_get_signal_queue(SdBusObject* self, PyObject* const* args, Py_ssize_t nargs) {
         SD_BUS_PY_CHECK_ARGS_NUMBER(4);
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(1, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(2, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(3, PyUnicode_Type);
 
-        const char* sender_service_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[0]);
-        const char* path_name_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[1]);
-        const char* interface_name_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[2]);
-        const char* member_name_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[3]);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, _unicode_or_none);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(1, _unicode_or_none);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(2, _unicode_or_none);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(3, _unicode_or_none);
+
+        const char* sender_service_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR_OPTIONAL(args[0]);
+        const char* path_name_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR_OPTIONAL(args[1]);
+        const char* interface_name_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR_OPTIONAL(args[2]);
+        const char* member_name_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR_OPTIONAL(args[3]);
 #else
 static PyObject* SdBus_get_signal_queue(SdBusObject* self, PyObject* args) {
         const char* sender_service_char_ptr = NULL;
@@ -430,9 +443,9 @@ static PyObject* SdBus_get_signal_queue(SdBusObject* self, PyObject* args) {
         const char* interface_name_char_ptr = NULL;
         const char* member_name_char_ptr = NULL;
         CALL_PYTHON_BOOL_CHECK(
-            PyArg_ParseTuple(args, "ssss", &sender_service_char_ptr, &path_name_char_ptr, &interface_name_char_ptr, &member_name_char_ptr, NULL));
+            PyArg_ParseTuple(args, "zzzz", &sender_service_char_ptr, &path_name_char_ptr, &interface_name_char_ptr, &member_name_char_ptr, NULL));
 #endif
-        SdBusSlotObject* new_slot CLEANUP_SD_BUS_SLOT = (SdBusSlotObject*)CALL_PYTHON_AND_CHECK(PyObject_CallFunctionObjArgs(SdBusSlot_class, NULL));
+        SdBusSlotObject* new_slot CLEANUP_SD_BUS_SLOT = (SdBusSlotObject*)CALL_PYTHON_AND_CHECK(SD_BUS_PY_CLASS_DUNDER_NEW(SdBusSlot_class));
 
         PyObject* new_queue CLEANUP_PY_OBJECT = CALL_PYTHON_AND_CHECK(PyObject_CallFunctionObjArgs(asyncio_queue_class, NULL));
 
@@ -450,7 +463,7 @@ static PyObject* SdBus_get_signal_queue(SdBusObject* self, PyObject* args) {
                                                         interface_name_char_ptr, member_name_char_ptr, _SdBus_signal_callback,
                                                         _SdBus_match_signal_instant_callback, new_future));
 
-        CHECK_SD_BUS_READER
+        CHECK_SD_BUS_READER;
         Py_INCREF(new_future);
         return new_future;
 }
@@ -484,8 +497,8 @@ int SdBus_request_callback(sd_bus_message* m,
 #ifndef Py_LIMITED_API
 static PyObject* SdBus_request_name_async(SdBusObject* self, PyObject* const* args, Py_ssize_t nargs) {
         SD_BUS_PY_CHECK_ARGS_NUMBER(2);
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(1, PyLong_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(1, PyLong_Check);
 
         const char* service_name_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[0]);
         uint64_t flags = PyLong_AsUnsignedLongLong(args[1]);
@@ -501,7 +514,7 @@ static PyObject* SdBus_request_name_async(SdBusObject* self, PyObject* args) {
 #endif
         PyObject* running_loop CLEANUP_PY_OBJECT = CALL_PYTHON_AND_CHECK(PyObject_CallFunctionObjArgs(asyncio_get_running_loop, NULL));
         PyObject* new_future = CALL_PYTHON_AND_CHECK(PyObject_CallMethod(running_loop, "create_future", ""));
-        SdBusSlotObject* new_slot_object CLEANUP_SD_BUS_SLOT = (SdBusSlotObject*)CALL_PYTHON_AND_CHECK(PyObject_CallFunctionObjArgs(SdBusSlot_class, NULL));
+        SdBusSlotObject* new_slot_object CLEANUP_SD_BUS_SLOT = (SdBusSlotObject*)CALL_PYTHON_AND_CHECK(SD_BUS_PY_CLASS_DUNDER_NEW(SdBusSlot_class));
 
         CALL_SD_BUS_AND_CHECK(
             sd_bus_request_name_async(self->sd_bus_ref, &new_slot_object->slot_ref, service_name_char_ptr, flags, SdBus_request_callback, new_future));
@@ -516,8 +529,8 @@ static PyObject* SdBus_request_name_async(SdBusObject* self, PyObject* args) {
 #ifndef Py_LIMITED_API
 static PyObject* SdBus_request_name(SdBusObject* self, PyObject* const* args, Py_ssize_t nargs) {
         SD_BUS_PY_CHECK_ARGS_NUMBER(2);
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);
-        SD_BUS_PY_CHECK_ARG_TYPE(1, PyLong_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(1, PyLong_Check);
 
         const char* service_name_char_ptr = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[0]);
         uint64_t flags = PyLong_AsUnsignedLongLong(args[1]);
@@ -538,7 +551,7 @@ static PyObject* SdBus_request_name(SdBusObject* self, PyObject* args) {
 #ifndef Py_LIMITED_API
 static SdBusSlotObject* SdBus_add_object_manager(SdBusObject* self, PyObject* const* args, Py_ssize_t nargs) {
         SD_BUS_PY_CHECK_ARGS_NUMBER(1);
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);
 
         const char* object_manager_path = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[0]);
 #else
@@ -546,7 +559,7 @@ static SdBusSlotObject* SdBus_add_object_manager(SdBusObject* self, PyObject* ar
         const char* object_manager_path = NULL;
         CALL_PYTHON_BOOL_CHECK(PyArg_ParseTuple(args, "s", &object_manager_path, NULL));
 #endif
-        SdBusSlotObject* new_slot_object CLEANUP_SD_BUS_SLOT = (SdBusSlotObject*)CALL_PYTHON_AND_CHECK(PyObject_CallFunctionObjArgs(SdBusSlot_class, NULL));
+        SdBusSlotObject* new_slot_object CLEANUP_SD_BUS_SLOT = (SdBusSlotObject*)CALL_PYTHON_AND_CHECK(SD_BUS_PY_CLASS_DUNDER_NEW(SdBusSlot_class));
 
         CALL_SD_BUS_AND_CHECK(sd_bus_add_object_manager(self->sd_bus_ref, &new_slot_object->slot_ref, object_manager_path));
 
@@ -557,7 +570,7 @@ static SdBusSlotObject* SdBus_add_object_manager(SdBusObject* self, PyObject* ar
 #ifndef Py_LIMITED_API
 static PyObject* SdBus_emit_object_added(SdBusObject* self, PyObject* const* args, Py_ssize_t nargs) {
         SD_BUS_PY_CHECK_ARGS_NUMBER(1);
-        SD_BUS_PY_CHECK_ARG_TYPE(0, PyUnicode_Type);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);
 
         const char* added_object_path = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[0]);
 #else
@@ -570,6 +583,32 @@ static PyObject* SdBus_emit_object_added(SdBusObject* self, PyObject* args) {
         Py_RETURN_NONE;
 }
 
+#ifndef Py_LIMITED_API
+static PyObject* SdBus_emit_object_removed(SdBusObject* self, PyObject* const* args, Py_ssize_t nargs) {
+        SD_BUS_PY_CHECK_ARGS_NUMBER(1);
+        SD_BUS_PY_CHECK_ARG_CHECK_FUNC(0, PyUnicode_Check);
+
+        const char* removed_object_path = SD_BUS_PY_UNICODE_AS_CHAR_PTR(args[0]);
+#else
+static PyObject* SdBus_emit_object_removed(SdBusObject* self, PyObject* args) {
+        const char* removed_object_path = NULL;
+        CALL_PYTHON_BOOL_CHECK(PyArg_ParseTuple(args, "s", &removed_object_path, NULL));
+#endif
+        CALL_SD_BUS_AND_CHECK(sd_bus_emit_object_removed(self->sd_bus_ref, removed_object_path));
+
+        Py_RETURN_NONE;
+}
+
+static PyObject* SdBus_close(SdBusObject* self, PyObject* Py_UNUSED(args)) {
+        sd_bus_close(self->sd_bus_ref);
+        Py_RETURN_NONE;
+}
+
+static PyObject* SdBus_start(SdBusObject* self, PyObject* Py_UNUSED(args)) {
+        CALL_SD_BUS_AND_CHECK(sd_bus_start(self->sd_bus_ref));
+        Py_RETURN_NONE;
+}
+
 static PyMethodDef SdBus_methods[] = {
     {"call", (SD_BUS_PY_FUNC_TYPE)SdBus_call, SD_BUS_PY_METH, "Send message and get reply"},
     {"call_async", (SD_BUS_PY_FUNC_TYPE)SdBus_call_async, SD_BUS_PY_METH, "Async send message, returns awaitable future"},
@@ -589,9 +628,29 @@ static PyMethodDef SdBus_methods[] = {
     {"request_name", (SD_BUS_PY_FUNC_TYPE)SdBus_request_name, SD_BUS_PY_METH, "Request dbus name blocking"},
     {"add_object_manager", (SD_BUS_PY_FUNC_TYPE)SdBus_add_object_manager, SD_BUS_PY_METH, "Add object manager at the path"},
     {"emit_object_added", (SD_BUS_PY_FUNC_TYPE)SdBus_emit_object_added, SD_BUS_PY_METH, "Emit signal that object was added"},
+    {"emit_object_removed", (SD_BUS_PY_FUNC_TYPE)SdBus_emit_object_removed, SD_BUS_PY_METH, "Emit signal that object was removed"},
+    {"close", (PyCFunction)SdBus_close, METH_NOARGS, "Close connection"},
+    {"start", (PyCFunction)SdBus_start, METH_NOARGS, "Start connection"},
     {NULL, NULL, 0, NULL},
 };
 
+static PyObject* SdBus_address_getter(SdBusObject* self, void* Py_UNUSED(closure)) {
+        const char* bus_address = NULL;
+        int get_address_result = sd_bus_get_address(self->sd_bus_ref, &bus_address);
+        if (-ENODATA == get_address_result) {
+                // Bus has not been set yet
+                Py_RETURN_NONE;
+        } else {
+                CALL_SD_BUS_AND_CHECK(get_address_result);
+        }
+        return PyUnicode_FromString(bus_address);
+}
+
+static PyGetSetDef SdBus_properies[] = {
+    {"address", (getter)SdBus_address_getter, NULL, "Bus address", NULL},
+    {0},
+};
+
 PyType_Spec SdBusType = {
     .name = "sd_bus_internals.SdBus",
     .basicsize = sizeof(SdBusObject),
@@ -599,9 +658,11 @@ PyType_Spec SdBusType = {
     .flags = Py_TPFLAGS_DEFAULT,
     .slots =
         (PyType_Slot[]){
+            {Py_tp_new, PyType_GenericNew},
             {Py_tp_init, (initproc)SdBus_init},
             {Py_tp_dealloc, (destructor)SdBus_dealloc},
             {Py_tp_methods, SdBus_methods},
+            {Py_tp_getset, SdBus_properies},
             {0, NULL},
         },
 };
